/*
 *
 *	Class(ses):	RoutingTransportPPE
 *	Revision:	$Id: AddressingPPE.h,v 1.7 2005/10/04 09:11:17 marcin Exp $
 *	Author:	 Marcin Brzozowski, Tobias Senner
 *
 *	Description:	A ppe with transport and routing
 *			
 *	Note:		
 *			
 *
 *	(c) Steffen Schulze, BTU Cottbus 2006
 */

#ifndef copra_RoutingTransportCompPPE_h
#define copra_RoutingTransportCompPPE_h


#include "copra/pps/Reliability.h"
#include "copra/pps/RxDecoupler.h"

#include "copra/pps/AddressingStage.h"

#include "adt/Pool.h"
#include "copra/pps/NetBuffer.h"
#include "adt/GenericFiFo.h"
#include "copra/pps/Packet.h"

#include "reflex/timer/Clock.h"
#include "reflex/timer/Time.h"

using adt::Pool;
using adt::GenericFiFo;

using reflex::Clock;


namespace copra {

	/**
	 *	
	 *	@author Reinhardt Karnapke
	 *	@version $Id: AddressingPPE.h,v 1.7 2005/10/04 09:11:17 marcin Exp $
	 */


	class RoutingTransportCompPPE : public AddressingStage<NetAddress, NetBuffer> , public RxStage<NetBuffer> {
		public: 


			/**
			 *	Constructor. Here all parts of the ppe are assembled.
			 */

			RoutingTransportCompPPE(GenericPool* pool, NetAddress myId, ITxStage<NetBuffer>* ppeEnd, ITxStage<NetBuffer>* broadcastRMI) :  reliability(pool, myId,broadcastRMI)
			{
				this->pool = pool;

				reliability.txConnect(ppeEnd);
				
				
				rxConnect(&rxDecoupler);
				rxDecoupler.rxConnect(&reliability);
			}
			
			virtual void deliver(const NetAddress& n, NetBuffer* stack)
			{
				stack->push(n);
				reliability.deliver(stack);
			}

			void setID(int n)
			{
				id = n;
			}

			void setBroadcastID(int n)
			{
				broadcastID = n;
			}

			virtual void accept(NetBuffer* stack)
			{
				rxForward(stack);
			}

			void setClock(Clock* cl)	
			{ 
				clock = cl; 
			}

			void setPool(GenericPool* pool) 	
			{ 
				this->pool = pool;
			}

			// the elements of the ppe
			TxStage<NetBuffer>* nextTxStage;
			
			Reliability reliability;
			RxDecoupler<10> rxDecoupler;
			//ReliabilityCopies reliability;
			
			
			
			int id;
			int broadcastID;

			Clock* clock;	// needed by transport for timeouts
			GenericPool* pool;	

	};

} //namespace copra
#endif
