/*
 *
 *	Class(ses):	Compression
 *	Revision:	$Id:$
 *	Author:	 Steffen Schulze
 *
 *	Description:	The merged version of the Compression Rx and Tx paths of the pps
 *			
 *	Note:		
 *			
 *
 *	(c) Steffen Schulze, BTU Cottbus 2005
 */

#include "copra/pps/Compression.h"
#include "copra/pps/Packet.h"

#include "conf.h"
//#include "std/string.h"
#include "copra/pps/compress/CompressConfig.h"

namespace copra {

void Compression::accept(NetBuffer* stack)
{
    process(stack, false);
    rxForward(stack);
}
void Compression::deliver(NetBuffer* stack)
{
    process(stack, true);
    txForward(stack);
}

using std::memcpy;

#ifdef DEBUG_COMPRESS_DETAIL
using std::cerr;
using std::endl;
#endif


void Compression::process(NetBuffer* stack, bool encode)
{
	{//DEBUG
		#ifdef DEBUG_COMPRESS
		if(encode)
		{
    		    debug.inform(EVENT_COMPRESS_TXSTART + stack->used(), "> TxCompression::deliver()");
		}
		else
		{
    		    debug.inform(EVENT_COMPRESS_RXSTART + stack->used(), "> RxCompression::accept()");
		}
		#endif
	}
	

	//////////////
	// De-/Encode	
	
	CODEC_CODECS //defined in CompressConfig.h

	CompressorCodec* codec;

	UINTMAX codedSize;
	UINTMAX srcCacheSize;

	BYTE* destCache;
	BYTE* srcCache;
	
	BYTE dummy[PacketPayload];
	BYTE dummy2[PacketPayload];
	destCache = (BYTE*)(&dummy);
	srcCache = (BYTE*)(&dummy2);
	
	srcCacheSize = stack->used();

	memcpy(srcCache,stack->tos(),stack->used());
	
	#ifdef DEBUG_COMPRESS_DETAIL
	cerr << endl << "\tsrcCacheSize(" << srcCacheSize << "):(" << (int)srcCache << ")" << endl;
	for(UINT16 i = 0; i < srcCacheSize; i++)
	{
		cerr << i << ":(" << srcCache[i] << ")" << endl;
	}
	#endif
	
	//for only one compressionstage an alternative path ist possible.. for less codesize	
	for(BYTE i = 0; i < CODEC_COUNT; i++)
	{
	    codec = codecs[(encode)?(i):(CODEC_COUNT - i - 1)];
	
		#ifdef DEBUG_COMPRESS_DETAIL
		cerr << "run step : (" << (int)i << ")" << endl;
		#endif
		
		if(encode)
		{
			codedSize = codec->encode(srcCache, srcCacheSize, destCache, PacketPayload);
		}
	    else
		{
			codedSize = codec->decode(srcCache, srcCacheSize, destCache, PacketPayload);
		}
	
		//error while coding
		if(!codedSize)
		{
		    {//DEBUG
			#ifdef DEBUG_COMPRESS
			if(encode)
			{
			    debug.inform(EVENT_COMPRESS_TXERROR,"! TxCompression::deliver() - ERROR WHILE COMPRESSING!");
			}
			else
			{
			    debug.inform(EVENT_COMPRESS_RXERROR,"! RxCompression::accept() - ERROR WHILE DECOMPRESSING!");
			}
			#endif
		    }
		    return;
	    }

		BYTE* c	= srcCache;
		srcCache	= destCache;
		destCache	= c;
		
		srcCacheSize = codedSize;
	
		{//DEBUG
			#ifdef DEBUG_COMPRESS_DETAIL
			cerr << endl << "\tcodedSize(" << codedSize << ")" << endl;
			for(UINT16 i = 0; i < srcCacheSize; i++)
			{
				cerr << i << ":(" << srcCache[i] << ") " << endl;
			}
			cerr << "EOF" << endl << endl;
			#endif
		}
	
	
	}
	
	//
	// De-/Encode	
	//////////////
		
	stack->reset();
	stack->alloc(srcCacheSize);
	
	memcpy(stack->tos(),srcCache,srcCacheSize);

	
	{//DEBUG
		#ifdef DEBUG_COMPRESS_DETAIL
		cerr << "FINAL(" << srcCacheSize << "):(" << (int)srcCache << ")" << endl;
		for(UINT16 i = 0; i < stack->used(); i++)
		{
			cerr << (i) << ":(" << ((BYTE*)(stack->tos()))[i] << ") " << endl;
		}
		#endif
	}
	
	
	{//DEBUG
		#ifdef DEBUG_COMPRESS
		if(encode)
		{
		    debug.inform(EVENT_COMPRESS_TXFINISH + stack->used(), "< TxCompression::deliver()");
		}
		else
		{
		    debug.inform(EVENT_COMPRESS_RXFINISH + stack->used(), "< RxCompression::accept()");
		}
		#endif
	}
}

} //namespace
