/*
 *
 *	Class(ses):	Compression
 *	Revision:	$Id:$
 *	Author:	 Steffen Schulze
 *
 *	Description:	The merged version of the Compression Rx and Tx paths of the pps
 *			
 *	Note:		
 *			
 *
 *	(c) Steffen Schulze, BTU Cottbus 2005
 */

#include "copra/pps/Compression.h"
#include "copra/pps/Packet.h"


namespace copra {

UINTMAX Compression::process(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize, bool encode)
{

#ifdef COMPRESS_COMPRESS_ON	

	//////////////
	// De-/Encode	
	
	COMPRESS_CODEC_CODECS //defined in CompressConfig.h

	CompressorCodec* codec;

	UINTMAX srcCacheSize = inputBufferSize;
	UINTMAX destCacheSize;

	BYTE* destCache;
	BYTE* srcCache;
	
	BYTE dummy[COMPRESS_BUFFER_MAXSIZE];
	BYTE dummy2[COMPRESS_BUFFER_MAXSIZE];
	destCache = (BYTE*)(dummy);
	srcCache = (BYTE*)(dummy2);
	
	memcpy(srcCache, inputBuffer, inputBufferSize);

	//for only one compressionstage an alternative path iss possible.. for less codesize	
	for(BYTE i = 0; i < COMPRESS_CODEC_COUNT; i++)
	{
		codec = codecs[(encode)?(i):(COMPRESS_CODEC_COUNT - i - 1)];
	
		{//DEBUG
			#ifdef COMPRESS_DEBUG_DETAIL
			cerr << "Compression::process() run step : (" << (int)i << ")" << endl;
			#endif
		}
		
		if(encode)
		{
			destCacheSize = codec->encode(srcCache, srcCacheSize, destCache, COMPRESS_BUFFER_MAXSIZE);
		}
		else
		{
			destCacheSize = codec->decode(srcCache, srcCacheSize, destCache, COMPRESS_BUFFER_MAXSIZE);
		}
	
		//error while coding
		if(!destCacheSize)
		{
		    {//DEBUG
				#ifdef COMPRESS_DEBUG
				if(encode)
				{
					debug.inform(EVENT_COMPRESS_TXERROR + COMPRESS_ACTIVENODE,"! TxCompression::process() - ERROR WHILE COMPRESSING!");
					return 0;
				}
				else
				{
					debug.inform(EVENT_COMPRESS_RXERROR + COMPRESS_ACTIVENODE,"! RxCompression::process() - ERROR WHILE DECOMPRESSING!");
					return 0;
				}
				#endif
			}
		}
		
		{//DEBUG
			#ifdef COMPRESS_DEBUG_DETAIL
			cerr << "Compression::process() destCacheSize(" << destCacheSize << ")" << endl;
			for(UINT16 i = 0; i < destCacheSize; i++)
			{
				cerr << i << ":(" << (unsigned)(destCache[i]) << ") " << endl;
			}
			cerr << "EOF"<< endl;
			#endif
		}
		
		BYTE* c	= srcCache;
		srcCache	= destCache;
		destCache	= c;
		
		srcCacheSize = destCacheSize;
	}
	
	//
	// De-/Encode	
	//////////////


	if(srcCacheSize > outputBufferSize)
	{
		return 0;
	}
	
	memcpy(outputBuffer, srcCache , srcCacheSize);

	return srcCacheSize;

#else //COMPRESS_COMPRESS_ON
	if(inputBufferSize > outputBufferSize)
	{
		return 0;
	}
	memcpy(outputBuffer, inputBuffer , inputBufferSize);

	
	return inputBufferSize;
#endif //COMPRESS_COMPRESS_ON
}



} //namespace
