/*
 *
 *	Class(ses):	Compression
 *	Revision:	$Id:$
 *	Author:	 Steffen Schulze
 *
 *	Description:	The merged version of the Compression Rx and Tx paths of the pps
 *			
 *	Note:		
 *			
 *
 *	(c) Steffen Schulze, BTU Cottbus 2005
 */

#include "copra/pps/Compression.h"
#include "copra/pps/Packet.h"


namespace copra {

#ifdef COMPRESS_MERGE_ON
NetBuffer* Compression::merge(NetBuffer* newStack)
{
	
	{//DEBUG
		#ifdef COMPRESS_DEBUG
		debug.inform(EVENT_COMPRESS_MERGE + mergedStack.used() + COMPRESS_ACTIVENODE, "- TxCompression::merge()   - merge");
		#endif
	}
	
	//if empty stack
	if( !(newStack->used()) )
	{
		return 0;
	}
	
	//enter merging state
	isMerging = true;
	
	//if uncompressed mergedStack is to full to add newStack -> flush
	if(!((newStack->used() + mergedStack.used() + 1) < mergedStack.size()))
	{
		{//DEBUG
			#ifdef COMPRESS_DEBUG
			debug.inform(EVENT_COMPRESS_MERGELOWBUF + COMPRESS_ACTIVENODE , "- TxCompression::merge()   - buffer to small");
			#endif
		}
	}
	else
	{
		//add newStack to mergedStack
		appendStack(&mergedStack, newStack);
		
		if(mergedStack.used() <= COMPRESS_BUFFER_PACKETSIZE)
		{
			//no need for the newStack anymore
			newStack->downRef();
			
			if(mergedNetBuffer)
			{//compressed mergedNetBuffer is not empty
				NetBuffer* cachedNetBuffer = mergedNetBuffer;
				mergedNetBuffer = 0;
				cachedNetBuffer->downRef();
			}
			#ifdef COMPRESS_TIMEOUT_ON
			else
			{//mergedNetBuffer is empty
				//start timeout
				{//DEBUG
					#ifdef COMPRESS_DEBUG
					debug.inform(EVENT_COMPRESS_TIMERRESET + COMPRESS_ACTIVENODE, "- TxCompression::merge()   - TimeOutStart()");
					#endif
				}
				timer.start(COMPRESS_TIMEOUT_FLUSH);
			}
			#endif
			
			mergedNetBuffer = createNetBuffer((BYTE*)(mergedStack.tos()), mergedStack.used());
			
			//uncompressed flag
			mergedNetBuffer->push(false);
			
			{//DEBUG
				#ifdef COMPRESS_DEBUG
				countMerged++;
				debug.inform(EVENT_COMPRESS_MERGESUCU + COMPRESS_ACTIVENODE + mergedNetBuffer->used(), "- TxCompression::merge()   - successful merged uncomp");
				#endif
			}
			
			//leave mergingstate
			isMerging = false;
			return 0;
		}
		else
		{
			
			//some buffering because negative growing stack
			BYTE	dummy[COMPRESS_BUFFER_MAXSIZE];
			BYTE*	cacheBuffer = dummy;
			BYTE	cacheBufferSize;
		
			//try to compress the mergedStack
			if(cacheBufferSize = process((BYTE*)(mergedStack.tos()), mergedStack.used(), cacheBuffer, COMPRESS_BUFFER_PACKETSIZE , true))
			{//successful
				//no need for the newStack anymore
				newStack->downRef();
				
				if(mergedNetBuffer)
				{//compressed mergedNetBuffer is not empty
					NetBuffer* cachedNetBuffer = mergedNetBuffer;
					mergedNetBuffer = 0;
					cachedNetBuffer->downRef();
				}
				#ifdef COMPRESS_TIMEOUT_ON
				else
				{//compressed mergedNetBuffer is empty
					//start timeout
					{//DEBUG
						#ifdef COMPRESS_DEBUG
						debug.inform(EVENT_COMPRESS_TIMERRESET + COMPRESS_ACTIVENODE, "- TxCompression::merge()   - TimeOutStart()");
						#endif
					}
					timer.start(COMPRESS_TIMEOUT_FLUSH);
				}
				#endif
				
				mergedNetBuffer = createNetBuffer(cacheBuffer, cacheBufferSize);
				//compressed flag
				mergedNetBuffer->push(true);
				
				{//DEBUG
					#ifdef COMPRESS_DEBUG
					countMerged++;
					debug.inform(EVENT_COMPRESS_MERGESUCC + COMPRESS_ACTIVENODE + mergedNetBuffer->used(), "- TxCompression::merge()   - successful merged comp");
					#endif
				}
				
				isMerging = false;
				return 0;
			}
		
		}//clean stack
	
		//clean newStack from mergedStack
		BYTE newStackSize;
		mergedStack.pop(newStackSize);
		mergedStack.decreaseSP(newStackSize);
	}

	if(!mergedNetBuffer && mergedStack.used())
	{//if no ready packet -> create one
		mergedNetBuffer = createNetBuffer((BYTE*)(mergedStack.tos()), mergedStack.used());
		//uncompressed flag
		mergedNetBuffer->push(false);
	}

	{//DEBUG
		#ifdef COMPRESS_DEBUG
		debug.inform(EVENT_COMPRESS_MERGEFULL + COMPRESS_ACTIVENODE + mergedNetBuffer->used(), "- TxCompression::merge()   - begin new merge");
		#endif
	}

	isMerging = false;
	flush();
	isMerging = true;

	appendStack(&mergedStack, newStack);
	//no more need for packet
	newStack->downRef();

	//leave mergingstate
	isMerging = false;
	
	return mergedNetBuffer;
}

NetBuffer* Compression::unmerge(NetBuffer* stack)
{
	
	stack->used();
	
	{//DEBUG
			#ifdef COMPRESS_DEBUG
			debug.inform(EVENT_COMPRESS_UNMERGE + stack->used() + COMPRESS_ACTIVENODE, "> RxCompression::unmerge() - unmerge");
			#endif
	}
	
	
	bool isCompressed;
	stack->pop(isCompressed);
	
	Stack* _stack = stack;
	
	stack->used();
		
	BYTE	cacheBuffer[COMPRESS_BUFFER_MAXSIZE];
	BYTE	cacheBufferSize = 0;
	
	if(isCompressed)
	{
		if(!(cacheBufferSize = process((BYTE*)stack->tos(), stack->used(), cacheBuffer, COMPRESS_BUFFER_MAXSIZE, false)))
		{
			{//DEBUG
				#ifdef COMPRESS_DEBUG_DETAIL
				cerr << "\tCompression::accept() PANIC" << endl;
				#endif
			}
			return 0;
		}
		
		{//DEBUG
			#ifdef COMPRESS_DEBUG
			debug.inform(EVENT_COMPRESS_UNMERGEUC + cacheBufferSize + COMPRESS_ACTIVENODE, "> RxCompression::unmerge() - uncomp");
			#endif
		}
		
	}
	
	Stack	cacheStack(cacheBuffer, cacheBufferSize);

	if(isCompressed)
	{
		cacheStack.alloc(cacheBufferSize);
		_stack = &cacheStack;
		stack->downRef();
	}
	
	while(_stack->used())
	{//stack ist not empty

		BYTE outputStackSize;
		_stack->pop(outputStackSize);

		assert(outputStackSize);

		NetBuffer* outputStack = createNetBuffer((BYTE*)(_stack->tos()), outputStackSize);

		_stack->decreaseSP(outputStackSize);

		{//DEBUG
			#ifdef COMPRESS_DEBUG
			debug.inform(EVENT_COMPRESS_UNMERGEFLUSH + outputStack->used() + COMPRESS_ACTIVENODE, "< RxCompression::unmerge() - unmergeFlush");
			#endif
		}

		rxForward(outputStack);
	}
	
	stack->downRef();
	
	return 0;
}

void Compression::flush()
{
	
	if(mergedNetBuffer && !isMerging)
	{
		{//DEBUG
			#ifdef COMPRESS_DEBUG
		   	debug.inform(EVENT_COMPRESS_MERGEFLUSH + countMerged + COMPRESS_ACTIVENODE, "< TxCompression::flush()   - mergeFlush");
			#endif

			#ifdef COMPRESS_DEBUG
			debug.inform(EVENT_COMPRESS_TXFORWARD + COMPRESS_ACTIVENODE + mergedNetBuffer->used(), "< TxCompression::flush()   - FORWARD");
			countMerged = 0;
			#endif
		}
		
		//verhindern von mehrfach-forwards durch unterbrechendes notify
		NetBuffer* cache = mergedNetBuffer;
		mergedNetBuffer = 0;
		mergedStack.reset();
		
		//unmerge(cache, true);
		txForward(cache);
		
		
		#ifdef COMPRESS_TIMEOUT_ON
		{//DEBUG
			#ifdef COMPRESS_DEBUG
			debug.inform(EVENT_COMPRESS_TIMERRESET + COMPRESS_ACTIVENODE, "- TxCompression::flush()   - TimeOutStop()");
			#endif
		}
		timer.start(0);
		#endif
				
	}
}

#ifdef COMPRESS_TIMEOUT_ON
void Compression::notify()
{
	{//DEBUG
		#ifdef COMPRESS_DEBUG
		debug.inform(EVENT_COMPRESS_TIMERFLUSH + COMPRESS_ACTIVENODE + countMerged, "- TxCompression::notify()  - mergetimeout -> flush");
		#endif
	}

	eventFlush.notify();
}
#endif //COMPRESS_TIMEOUT_ON

#endif //COMPRESS_MERGE_ON
} //namespace
