#ifndef COMPRESSORCODEC_H
#define COMPRESSORCODEC_H

/*
 * REFLEX - Real-time Event FLow EXecutive
 *
 * A lightweight operating system for deeply embedded systems.
 *
 *
 * Class(ses): BwtCodec
 * Revision: $Id:$
 * Author: Steffen Schulze
 *
 * Description: Baseclass for compressioncodecs
 *
 * (c) Steffen Schulze, BTU Cottbus 2005
 */

#include "config.h"

class CompressorCodec
{
	public:
		CompressorCodec()
		{
		}
	
		/**
		 * Encode-method
		 *
		 * Virtual method for encoding. Have to be implemented in
		 * the class which inherits from this.
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual UINTMAX encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize) = 0;
		
		/**
		 * Decode-method
		 *
		 * Virtual method for decoding. Have to be implemented in
		 * the class which inherits from this.
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual UINTMAX decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize) = 0;
};


#endif
