#ifndef BWTCODEC_H
#define BWTCODEC_H

/*
 * REFLEX - Real-time Event FLow EXecutive
 *
 * A lightweight operating system for deeply embedded systems.
 *
 *
 * Class(ses): BwtCodec
 * Revision: $Id:$
 * Author: Steffen Schulze
 *
 * Description: Codec that implements Burrows-Wheeler-Transformation
 *
 * (c) Steffen Schulze, BTU Cottbus 2005
 */

#include "compress/CompressorCodec.h"

/**
 * Implementation of the Burrows-Wheeler-Transformation.
 * Sorts the inputstring by the charactervalues and displace each
 * character in sortet string by its following character from the
 * inputstring. first bits in outputdata gives the index for the
 * first character of inputmessage.
 * for decoding the algorithm sorts the data again by charactervalues.
 * transformate it back to the inputmessage with help of the index,
 * which is given by the first bits of transformed data.
 *
 * @author Steffen Schulze
 * @version $Id$
 */
class BwtCodec : public CompressorCodec
{
	public:
		/**
		 * Constructor
		 *
		 * Calculates the bitlength of the sizeinfo in the dataheader.
		 */
		BwtCodec()
		{
		}
		
		/**
		 * Encode-method
		 *
		 * Implements the virtual method encode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
		
		/**
		 * Decode-method
		 *
		 * Implements the virtual method decode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
	
	public:
		/**
		* Comparefunction for the Sortingalgorithm
		*
		* Have to be static, that the pointer to the function can send to the sortingalgorithm.
		* @param li the current index of left end
		* @param ri the current index of right end
		* @return @c 0 if both strings are equal @ !0 if both string are not equal
		*/
		static int compareFunction(UINTMAX li, UINTMAX ri);

		
		/**
		* The following static membervariables are for the compareFunction, because there is no
		* other efficient way to pass this informations from the encode- and decode-methods to the function.
		*/
	public:
		
		/**
		* Cachevariable for the functionvariable inputBuffer
		*/
		static BYTE* bwt_inputBuffer;
		
		/**
		* Cachevariable for the functionvariable inputBufferSize
		*/
		static UINTMAX bwt_inputBufferSize;
};
#endif
