#ifndef BITFIELD_H
#define BITFIELD_H

/*
 * REFLEX - Real-time Event FLow EXecutive
 *
 * A lightweight operating system for deeply embedded systems.
 *
 *
 * Class(ses): BitField
 * Revision: $Id:$
 * Author: Steffen Schulze
 *
 * Description: Datatype to write and read bitwise
 *
 * (c) Steffen Schulze, BTU Cottbus 2005
 */

 #include "../config.h"
 

/**
 * This class implements bitwise read and write operations
 * on a byte field. There is no sizecheck in this class!
 *
 * @author Steffen Schulze
 * @version $Id$
 */
class BitField
{
	public:
		
		/*
		 * Constructor initialize the membervariable currBit and currPos
		 * with 0. The pointer for the byteArray comes from parameter.
		 *
		 * @param _byteArray the pointer to the byteArray which will be used
		 */
		BitField(BYTE* _byteArray) : currBit(0), currPos(0), byteArray(_byteArray)
		{
		}
		
		/*
		 * Reset the postitions to startvalues. No data will be deleted.
		 */
		void reset()
		{
			currBit = 0;
			currPos = 0;
			byteArray[currPos] = 0;
		}
		
		/*
		 * Get the current index position in the bytearray
		 *
		 * @return current index in bytearray
		 */
		UINTMAX getPos()
		{
			return currPos;
		}
		
		/*
		 * Get the current bitposition in the current byte
		 *
		 * @return current bitposition in byte
		 */
		BYTE getBitPos()
		{
			return currBit;
		}
				
		/**
		 * Writes a number of bits into the bitField
		 *
		 * @param bits the bits in a variable 
		 * @param bitCount the number of bits
		 * @return @c 0 if the string is not in dictionary, @c index of the string
		 */
		void writeBits(UINTMAX bits, BYTE bitCount);
	
		/**
		 * Read a number of bits from the bitField
		 *
		 * @param bitCount the number of bits
		 * @return the bits
		 */
		UINTMAX readBits(BYTE bitCount);
		
		/*
		 * Read a single bit
		 *
		 * @return the bit
		 */
		BYTE readBit()
		{
			return readBits(1);
		}
		
		
#ifdef TEXTOUTPUT
		void printAll()
		{
			fprintf(stderr, "\t\t\t");
			for(UINT16 i = 0; i < 20; i++)
			{
				fprintf(stderr, "%x",byteArray[i]);
			}
			fprintf(stderr, "\n");
		}
#endif
	
	protected:
		
		/*
		 * Current bitposition in byte
		 */
		BYTE currBit;
		
		/*
		 * Current position in bytearray
		 */
		UINTMAX currPos;
		
		/*
		 * Pointer to the bytearray
		 */
		BYTE* byteArray;
};

#endif
