#ifndef MERGESORT_H
#define MERGESORT_H

/*
 * REFLEX - Real-time Event FLow EXecutive
 *
 * A lightweight operating system for deeply embedded systems.
 *
 *
 * Function(s): mergeSort
 * Revision: $Id:$
 * Author: Steffen Schulze
 *
 * Description: Datatype to write and read bitwise
 *
 * (c) Steffen Schulze, BTU Cottbus 2005
 */

#include "../config.h"
 
enum {UP, DOWN};

/*
 * Sorting-algorithm that uses mergesort.
 *
 * @param inputArray array to sort
 * @param arraySize size of inputArray
 * @param compareFunction to compare two elements of inputArray
 */
template<typename T>
void mergeSort(T* inputArray, UINTMAX arraySize, int (*compareFunction)(T,T))
{
	
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\t\tSTART mergeSort(%d)\n", arraySize);
	#endif

	T cacheArray[arraySize];
	
	T* sourceArray 	= inputArray;
	T* destArray	= cacheArray;
	
	
	for(UINTMAX slotSize = 1; slotSize < arraySize; slotSize += slotSize)
	{
		BYTE sortDest = UP;
		for(UINTMAX left = 0; left < arraySize; left += (slotSize + slotSize))
		{
			
			UINTMAX right = left + slotSize + slotSize - 1;
			if(right >= arraySize)
			{
				right = arraySize - 1;
			}
			
			UINTMAX l = left;
			UINTMAX r = right;
			
			if(sortDest == UP)
			{
				for(UINTMAX i = left; i < right; i++)
				{
					destArray[i] = (compareFunction(sourceArray[l],sourceArray[r]) != 1) ? sourceArray[l++] : sourceArray[r--];
				}
				destArray[right] = sourceArray[l++];
			}
			else
			{
				for(UINTMAX i = right; i > left; i--)
				{
					destArray[i] = (compareFunction(sourceArray[l],sourceArray[r]) != 1) ? sourceArray[l++] : sourceArray[r--];
				}
				destArray[left] = sourceArray[l++];
			}
			
				
			
			sortDest = (sortDest == UP) ? DOWN : UP;
		}
		
		T* cache	= sourceArray;
		sourceArray	= destArray;
		destArray	= cache;
	}
	
	//anders loesen!
	if(destArray == inputArray)
	{
		for(UINTMAX i = 0; i < arraySize; i++)
		{
			inputArray[i] = sourceArray[i];
		}
	}
	
	#ifdef TEXTDEBUG
	fprintf(stderr, "\t\t\t\tFINISH mergeSort()\n");
	#endif
	
}


#endif
