#ifndef LZWCODEC_H
#define LZWCODEC_H

/*
 * REFLEX - Real-time Event FLow EXecutive
 *
 * A lightweight operating system for deeply embedded systems.
 *
 *
 * Class(ses): : LzwCodec
 * Revision: $Id:$
 * Author: Steffen Schulze
 *
 * Description: Codec that implements Lempel-Ziv-Welch
 *
 * (c) Steffen Schulze, BTU Cottbus 2005
 */

#include "compress/CompressorCodec.h"


/**
 * Implementation of the Lempel-Ziv-Welch-Algorithm
 * Uses a dictionary to encode and decode the message.
 * The dictionary will created dynamicly while the coding.
 *
 * @author Steffen Schulze
 * @version $Id$
 */
class LzwCodec : public CompressorCodec
{
	public:
		LzwCodec()
		{
			#ifdef TEXTOUTPUT
			textOutputMode = 1;
			#endif
		}
		/**
		 * Encode-method
		 *
		 * Implements the virtual method encode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
		
		/**
		 * Decode-method
		 *
		 * Implements the virtual method decode of the baseclass CompressionCodec
		 * @param inputBuffer is pointer to inputdata
		 * @param inputBufferSize is lenght of inputBuffer
		 * @param outputBuffer is pointer to memory for outputdata
		 * @param outputBufferSize is maximal lenght of outputdata
		 * @return length of outputBuffer
		 */
		virtual	UINTMAX decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize);
		
	protected:
		#ifdef TEXTOUTPUT
		int textOutputMode;
		#endif
	enum
	{
		/*
		 * Size of the dictonary in byte
		 */
		DIC_SIZE = MAX_STACKMEMORY
	};
		
};

#endif
