#include "compress/lzw/LzwCodec.h"
#include "compress/libs/BitField.h"
#include "compress/libs/Dictionary.h"

UINTMAX	LzwCodec::encode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize)
{
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tSTART encodeLZW(%d)\n",inputBufferSize);
	}
	#endif
	
	Dictionary<DIC_SIZE> dic;
	
	#ifdef TEXTDEBUG
	{//debug
		fprintf(stderr, "\t\t\tinit Dictionary ");
	}
	#endif
	dic.initForLZW();
	#ifdef TEXTDEBUG
	{//debug
		fprintf(stderr, "\t\t[OK] :%d:\n", dic.getSize());
	}
	#endif
	
	UINTMAX outputBufferPos = 0;
	UINTMAX inputBufferPos = 0;
	
	BitField bitFieldOutputBuffer(outputBuffer);
	bitFieldOutputBuffer.reset();
	
	BYTE indexBitLength = 8;
	UINT16 limitBitLength = 256;
	
	Vector2D<UINT16> result;
	BYTE* currInputBuffer = inputBuffer;
	
	while(inputBufferPos < inputBufferSize)
	{
		//search longest phrase in dictionary
		// a = index of phrase (+1 before correction because of errorcode)
		// b = length of phrase
		result = dic.searchPhrase(currInputBuffer, (inputBufferSize - inputBufferPos));
	
		#ifdef TEXTOUTPUT
		{//panic
			//if nothing found... panic
			if(result.a == 0)
			{
				fprintf(stderr, "!!PANIC... phrase not found in dictionary\n");
				return 0;
			}
		}
		#endif
		
		result.a--;
		
		
		#ifdef TEXTDEBUG
		{//debug
			fprintf(stderr, "\t\t\t:found Phrase :%d:",result.a);
			for(UINT16 i = 0; i < result.b; i++)
			{
				fprintf(stderr, "%d-",currInputBuffer[i]);
			}
			fprintf(stderr, "\n");
		}
		#endif
		
		//write index to output
		bitFieldOutputBuffer.writeBits(result.a, indexBitLength);
		#ifdef TEXTDEBUG
		{//debug
			fprintf(stderr, "\t\t\t\twriteIndex :%d:%d",result.a, indexBitLength);
			fprintf(stderr, "\t\t\t\tinputBufferPos :%d\n",inputBufferPos);
		}
		#endif
		
		outputBufferPos = bitFieldOutputBuffer.getPos() + 1;
		MEMORYCHECK(outputBufferPos + 3 > outputBufferSize)
		
		//if not end of inputBuffer
		if((inputBufferPos + result.b) < inputBufferSize)
		{
			if(!dic.addString(currInputBuffer, result.b + 1))
			{//if dictionary is full
				#ifdef TEXTDEBUG
				{//debug
					fprintf(stderr, "\t\t\t\t\tdictionary is full\n");
				}
				#endif
				
				dic.reset();
				dic.initForLZW();
				#ifdef TEXTDEBUG
				{//debug
					fprintf(stderr, "\t\t\t\t\t\tdictionary reset\t\t[OK]\n");
				}
				#endif
				
				dic.addString(currInputBuffer, result.b);
			}
			
			#ifdef TEXTDEBUG
			{//debug
				fprintf(stderr, "\t\t\t\t:addString:%d:%d:",dic.getLastIndex(),dic.getSize());
				for(UINT16 i = 0; i <= result.b; i++)
				{
					fprintf(stderr, "%d-",currInputBuffer[i]);
				}
				fprintf(stderr, "\n");
			}
			#endif
			
		}
		
		inputBufferPos	+= result.b;
		currInputBuffer += result.b;
			
		if(dic.getLastIndex() >= limitBitLength)
		{//if bitlength of index is nomore long enough
			#ifdef TEXTDEBUG
			{//debug
				fprintf(stderr, "\t\t\tindexBitLength from :%d: to :%d:\n",indexBitLength, indexBitLength + 1);
				fprintf(stderr, "\t\t\t\tdic.getLastIndex :%d: >= :%d: :%d\n",dic.getLastIndex(), limitBitLength);
			}
			#endif
			limitBitLength <<= 1;
			indexBitLength += 1;
		}
	}
	
	outputBufferPos = bitFieldOutputBuffer.getPos() + 1;
	
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tFINISH encodeLZW(%d)\n",outputBufferPos);
	}
	#endif
	
	return outputBufferPos;
}




UINTMAX LzwCodec::decode(BYTE* inputBuffer, UINTMAX inputBufferSize, BYTE* outputBuffer, UINTMAX outputBufferSize)
{
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tSTART decodeBWT(%d)\n",inputBufferSize);
	}
	#endif
	
	Dictionary<DIC_SIZE> dic;

	#ifdef TEXTDEBUG
	{//debug
		fprintf(stderr, "\t\t\tinit Dictionary ");
	}
	#endif
	dic.initForLZW();
	#ifdef TEXTDEBUG
	{//debug
		fprintf(stderr, "\t\t[OK] :%d:\n", dic.getSize());
	}
	#endif
	
	
	
	UINT16 outputBufferPos = 0;
	
	BitField bitFieldInputBuffer(inputBuffer);
	
	BYTE indexBitLength = 8;
	UINT16 limitBitLength = 256;
	
	BYTE* lastOutputBuffer = outputBuffer;
	UINT16 lastPhraseLength = 0;
	
	BYTE* currOutputBuffer = outputBuffer;
	
	while(bitFieldInputBuffer.getPos() < inputBufferSize - 1)
	{
		//getindex from input
		UINT16 dicIndex = bitFieldInputBuffer.readBits(indexBitLength);
		dicIndex++;
		#ifdef TEXTOUTPUT
		{//debug and safty
			#ifdef TEXTDEBUG
			fprintf(stderr, "\t\t\t:readIndex:%d:%d\n",dicIndex, indexBitLength);
			#endif
			
			if(dicIndex > (dic.getLastIndex() + 1))
			{
				fprintf(stderr, "\nPANIC! - inputIndex is to big\n");
				return 0;
			}
		}
		#endif
		
		//search index in dictionary
		UINT16 phraseLength = dic.getString(dicIndex, currOutputBuffer);
		#ifdef TEXTDEBUG
		{//debug
			fprintf(stderr, "\t\t\t\t:getString:%d:\n",dicIndex);
		}
		#endif
		
		MEMORYCHECK((outputBufferPos + dic.getMaxLength()) >= outputBufferSize)

		if(!phraseLength)
		{//index in dictionary not found
			//preset phrase beginning
			currOutputBuffer[0] = lastOutputBuffer[0];
			
			#ifdef TEXTDEBUG
			{//debug
				fprintf(stderr, "\t\t\t\t:not found index :%d:",dicIndex);
				fprintf(stderr, "\t\t\t\t:preConstruct Entry:\n");
			}
			#endif

			if(!lastPhraseLength)
			{//PANIC - was the first inputindex
				
				#ifdef TEXTOUTPUT
				{//debug
					fprintf(stderr, "!!PANIC... index not found in dictionary\n");
				}
				#endif
				return 0;
			}
		}
		
		
		if(lastPhraseLength)
		{//it is not the first inputindex
			if(!dic.addString(lastOutputBuffer, lastPhraseLength + 1))
			{//dictionary is full
					#ifdef TEXTDEBUG
					{//debug
						fprintf(stderr, "\t\t\t\t\tdictionary is full\n");
					}
					#endif
					
					dic.reset();
					dic.initForLZW();
					#ifdef TEXTDEBUG
					{//debug
						fprintf(stderr, "\t\t\t\t\t\tdictionary reset\t\t[OK]\n");
					}
					#endif
					
					dic.addString(lastOutputBuffer, lastPhraseLength);
					phraseLength = 0;
			}
			
			#ifdef TEXTDEBUG
			{//debug
				fprintf(stderr, "\t\t\t\t:addString:%d:%d:",dic.getLastIndex(),dic.getSize());
				for(UINT16 i = 0; i <= lastPhraseLength; i++)
				{
					fprintf(stderr, "%d-",lastOutputBuffer[i]);
				}
				fprintf(stderr, "\n");
			}
			#endif
		}
		
	
		if(!phraseLength)
		{//index in dictionary not found
			//write phrase to output
			phraseLength = dic.getString(dicIndex, currOutputBuffer);
			
			#ifdef TEXTDEBUG
			{//debug
				fprintf(stderr, "\t\t\t\t:getString:%d: - again \n",dicIndex);
			}
			#endif
			
			#ifdef TEXTOUTPUT
			if(!phraseLength)
			{//PANIC - index not found
				fprintf(stderr, "!!PANIC... index not found in dictionary\n");
				return 0;
			}
			#endif

		}
			
		#ifdef TEXTDEBUG
		{//debug
			fprintf(stderr, "\t\t\t\t:found index :%d:",dicIndex);
			for(UINT16 i = 0; i < phraseLength; i++)
			{
				fprintf(stderr, "%d-",currOutputBuffer[i]);
			}
			fprintf(stderr, "\n");
		}
		#endif
		
		
		outputBufferPos += phraseLength;
		lastPhraseLength = phraseLength;
		lastOutputBuffer = currOutputBuffer;
		currOutputBuffer += phraseLength;
		
		#ifdef TEXTDEBUG
		{//debug
			fprintf(stderr, "\t\t\t\t\tdictionary size :%d:  outputBufferPos :%d:\n",dic.getLastIndex(), outputBufferPos);
		}
		#endif
		
		
		if((dic.getLastIndex() + 1) >= limitBitLength)
		{
			#ifdef TEXTDEBUG
			{//debug
				fprintf(stderr, "\t\t\tindexBitLength from :%d: to :%d:\n",indexBitLength, indexBitLength + 1);
				fprintf(stderr, "\t\t\t\tdic.getLastIndex :%d:+1 >= :%d: :%d\n",dic.getLastIndex(), limitBitLength);
			}
			#endif
			limitBitLength <<= 1;
			indexBitLength += 1;
		}
		
	}
	
	#ifdef TEXTOUTPUT
	if( textOutputMode )
	{
		fprintf(stderr, "\t\tFINISH decodeBWT(%d)\n",outputBufferPos);
	}
	#endif
		
	return outputBufferPos;
}


